@@ -62,11 +62,11 @@ _CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 @click.pass_context
 def cli(ctx, apiroot, config, organization):
     """A command line tool for working with Transcriptic."""
-    if ctx.invoked_subcommand in ['login']:
-        # Initialize empty connection
+    if ctx.invoked_subcommand in ['login', 'compile', 'preview', 'summarize', 'init']:
+        # For login/local commands, initialize empty connection
         ctx.obj = ContextObject()
         ctx.obj.api = Connection(use_environ=False)
-    elif ctx.invoked_subcommand not in ['compile', 'preview', 'summarize', 'init']:
+    else:
         try:
             ctx.obj = ContextObject()
             ctx.obj.api = Connection.from_file(config)
@@ -74,10 +74,17 @@ def cli(ctx, apiroot, config, organization):
                 ctx.obj.api.organization_id = organization
             if apiroot is not None:
                 ctx.obj.api.api_root = apiroot
-        except IOError:
-            click.echo("Error reading config file, running "
-                       "`transcriptic login` ...")
-            ctx.invoke(login)
+        except:
+            click.echo("Welcome to TxPy! It seems like your `.transcriptic` config file is missing or out of date")
+            analytics = click.confirm("Send TxPy CLI usage information to improve the CLI user "
+                                      "experience?", default=True)
+            ctx.obj.api = Connection(use_environ=False)  # Initialize empty connection
+            ctx.invoke(login, analytics=analytics)
+    if ctx.obj.api.analytics:
+        try:
+            ctx.obj.api._post_analytics(event_action=ctx.invoked_subcommand, event_category="cli")
+        except:
+            pass
 
 
 @cli.command()
@@ -92,7 +99,7 @@ def cli(ctx, apiroot, config, organization):
 @click.option('--title', '-t', help='Optional title of your run')
 @click.option('--test', help='Submit this run in test mode', is_flag=True)
 @click.pass_context
-def submit(ctx, file, project, title, test):
+def submit(ctx, file, project, title=None, test=None):
     """Submit your run to the project specified."""
     project = get_project_id(project)
     if not project:
@@ -229,22 +236,34 @@ def upload_release(ctx, archive, package):
 
 
 @cli.command()
-def protocols():
-    """List protocols within your manifest."""
-
-    manifest = load_manifest()
-    if 'protocols' not in list(manifest.keys()) or not manifest['protocols']:
-        click.echo("Your manifest.json file doesn't contain any protocols or"
-                   " is improperly formatted.")
-        return
+@click.pass_context
+@click.option(
+    '--remote',
+    is_flag=True,
+    required=False,
+    default=False,
+    help='Shows available protocols to be launched remotely'
+)
+def protocols(ctx, remote):
+    """List protocols within your manifest or organization."""
+    if remote:
+        protocol_objs = ctx.obj.api.get_protocols()
     else:
-        click.echo('\n{:^60}'.format("Protocols within this manifest:"))
-        click.echo('{:-^60}'.format(''))
-        [click.echo("%s%s\n%s" % (p['name'],
-                                  (" (" + p.get('display_name') + ")")
-                                  if p.get('display_name') else "",
-                                  ('{:-^60}'.format(""))))
-         for p in manifest["protocols"]]
+        manifest = load_manifest()
+        if 'protocols' not in list(manifest.keys()) or not manifest['protocols']:
+            click.echo("Your manifest.json file doesn't contain any protocols or"
+                       " is improperly formatted.")
+            return
+        protocol_objs = manifest['protocols']
+
+    click.echo('\n{:^60}'.format("Protocols within this {}:".format("organization" if remote else "manifest")))
+    click.echo('{:-^60}'.format(''))
+    for p in protocol_objs:
+        if p.get('display_name'):
+            display_str = "{} ({})".format(p['name'], p.get('display_name'))
+        else:
+            display_str = p['name']
+        click.echo("{}\n{}".format(display_str, '{:-^60}'.format("")))
 
 
 @cli.command()
@@ -474,6 +493,9 @@ def resources(ctx, query):
     resource_req = ctx.obj.api.resources(query)
     if resource_req["results"]:
         kit_req = ctx.obj.api.kits(query)
+        if not kit_req["results"]:
+            common_name = resource_req["results"][0]["name"]
+            kit_req = ctx.obj.api.kits(common_name)
         flat_items = list(flatmap(lambda x: [{"name": y["resource"]["name"],
                                               "id": y["resource"]["id"],
                                               "vendor": x["vendor"]["name"] if "vendor" in list(x.keys()) else ''}
@@ -504,6 +526,87 @@ def resources(ctx, query):
         click.echo("No results for '{}'.".format(query))
 
 
+@cli.command()
+@click.argument('query', default='*')
+@click.option('--include_aliquots', help='include containers with matching aliquots', is_flag=True)
+@click.option('--show_status', help='show container status', is_flag=True)
+@click.option('--retrieve_all', help='retrieve all samples, this may take a while', is_flag=True)
+@click.pass_context
+def inventory(ctx, include_aliquots, show_status, retrieve_all, query):
+    """Search organization for inventory"""
+    inventory_req = ctx.obj.api.inventory(query)
+    num_pages = inventory_req["num_pages"]
+    per_page = inventory_req["per_page"]
+    results = inventory_req["results"]
+    max_results_bound = num_pages * per_page
+
+    num_prefiltered = len(results)
+
+    if retrieve_all:
+        for i in range(1, num_pages):
+            click.echo("Retrieved {} records"
+                       " out of {} total for '{}'...\r".
+                       format(i * per_page,
+                              max_results_bound, query), nl=False)
+            inventory_req = ctx.obj.api.inventory(query, page=i)
+            results.extend(inventory_req["results"])
+        click.echo()
+
+    if include_aliquots:
+        results = [c if "label" in c else c["container"] for c in results]
+    else:
+        results = [c for c in results if "label" in c]
+    results = [i for n, i in enumerate(results) if i not in results[n + 1:]]
+
+    if results:
+        def truncate_time(d, k):
+            old_time = d[k]
+            d[k] = old_time.split("T")[0]
+            return d
+
+        results = [truncate_time(c, "created_at") for c in results]
+        barcode_present = any(c["barcode"] for c in results)
+        keys = ["label", "id", "container_type_id", "storage_condition", "created_at"]
+        if barcode_present:
+            keys.insert(2, "barcode")
+        if show_status:
+            keys.append("status")
+        friendly_keys = {k: k.split("_")[0] for k in keys}
+        spacing = {k: max(len(friendly_keys[k]),
+                          max([len(str(c[k])) for c in results])) for k in keys}
+        spacing = {k: (v // 2 + 1) * 2 + 1 for k, v in spacing.items()}
+        sum_spacing = sum(spacing.values()) + (len(keys) - 1) * 3 + 1
+        spacing = {k: "{:^%s}" % v for k, v in spacing.items()}
+        sum_spacing = "{:-^%s}" % sum_spacing
+        click.echo("Results for '%s':" % query)
+        click.echo(' | '.join([spacing[k].
+                              format(friendly_keys[k]) for k in keys]))
+        click.echo(sum_spacing.format(''))
+        for c in results:
+            click.echo(' | '.join([spacing[k].
+                                  format(ascii_encode(c[k])) for k in keys]))
+            click.echo(sum_spacing.format(''))
+        if not retrieve_all:
+            if num_pages > 1:
+                click.echo("Retrieved {} records out of "
+                           "{} total (use the --retrieve_all flag "
+                           "to request all records).".
+                           format(num_prefiltered, max_results_bound))
+    else:
+        if retrieve_all:
+            click.echo("No results for '{}'.".format(query))
+        else:
+            if num_pages > 1:
+                click.echo("Retrieved {} records out of "
+                           "{} total but all were filtered out. "
+                           "Use the --retrieve_all flag "
+                           "to request all records.".
+                           format(num_prefiltered, max_results_bound))
+            else:
+                click.echo("All records were filtered out. "
+                           "Use flags to modify your search")
+
+
 @cli.command()
 @click.argument('path', default='.')
 def init(path):
@@ -606,14 +709,16 @@ def preview(ctx, protocol_name, view):
 @click.pass_context
 @click.option('--tree', '-t', is_flag=True,
               help='Prints a job tree with instructions as leaf nodes')
+@click.option('--lookup', '-l', is_flag=True,
+              help='Queries Transcriptic to convert resourceID to string')
 # time allowance is on order of seconds
-@click.option('--runtime', "-r", type=click.INT, default=5)
-def summarize(ctx, file, tree, runtime):
+@click.option('--runtime', type=click.INT, default=5)
+def summarize(ctx, file, tree, lookup, runtime):
     """Summarize Autoprotocol as a list of plain English steps, as well as a
     visualized Job Tree contingent upon desired runtime allowance (in seconds).
     A Job Tree refers to a structure of protocol based on container dependency,
     where each node, and its corresponding number, represents an instruction of
-    the protocol. More specifically, the tree structure contians process branches,
+    the protocol. More specifically, the tree structure contains process branches,
     in which the x-axis refers to the dependency depth in a given branch, while 
     the y-axis refers to the traversal of branches themselves. 
 
@@ -630,7 +735,19 @@ def summarize(ctx, file, tree, runtime):
             click.echo(
                 "The autoprotocol you're trying to summarize is invalid.")
             return
-    parser = AutoprotocolParser(protocol)
+
+    if lookup:
+        try:
+            config = '~/.transcriptic'
+            ctx.obj = ContextObject()
+            ctx.obj.api = Connection.from_file(config)
+            parser = AutoprotocolParser(protocol, ctx=ctx)
+        except:
+            click.echo("Connection with Transcriptic failed. "
+                       "Summarizing without lookup.", err=True)
+            parser = AutoprotocolParser(protocol)
+    else:
+        parser = AutoprotocolParser(protocol)
 
     if tree:
         import multiprocessing
@@ -676,10 +793,16 @@ def compile(protocol_name, args):
 
 @cli.command()
 @click.argument('protocol')
+@click.argument(
+    'params',
+    metavar='PARAMETERS_FILE',
+    type=click.File('r'),
+    required=False
+)
 @click.option(
     '--project', '-p',
     metavar='PROJECT_ID',
-    required=True,
+    required=False,
     help='Project id or name context for configuring the protocol. Use \
          `transcriptic projects` command to list existing projects.'
 )
@@ -687,21 +810,132 @@ def compile(protocol_name, args):
     '--save_input',
     metavar='FILE',
     required=False,
-    help='Save the protocol input JSON in a file. This is useful for debugging \
-         a protocol.'
+    help='Save the protocol or parameters input JSON in a file. This is \
+          useful for debugging a protocol.'
+)
+@click.option(
+    '--remote',
+    is_flag=True,
+    required=False,
+    help='If specified, the protocol will execute remotely and submit a run. This is useful \
+          if you do not have the protocol locally, or are unable to execute it for any reason.'
 )
 @click.pass_context
-def launch(ctx, protocol, project, save_input):
-    """Configure and execute your protocol using your web browser to select
-     your inputs"""
-    project = get_project_id(project)
-    if not project:
-        return
+def launch(ctx, protocol, project, save_input, remote, params):
+    """Configure and launch a protocol either using the local manifest file or remotely.
+    If no parameters are specified, uses the webapp to select the inputs."""
 
-    manifest, protocol = load_manifest_and_protocol(protocol)
+    # Load protocol from local file if not remote and load from listed protocols otherwise
+    if not remote:
+        manifest, protocol_obj = load_manifest_and_protocol(protocol)
+    else:
+        print_stderr("Searching for {}...".format(protocol))
+        protocol_list = ctx.obj.api.get_protocols()
+        matched_protocols = [protocol_obj for protocol_obj in protocol_list if protocol_obj['name'] == protocol]
+        if len(matched_protocols) == 0:
+            print_stderr("Protocol {} was not found.".format(protocol))
+            return
+        elif len(matched_protocols) > 1:
+            print_stderr("More than one match found. Using the first match.")
+        else:
+            print_stderr("Protocol found.")
+        protocol_obj = matched_protocols[0]
 
+    # If parameters are not specified, use quick launch to get inputs
+    if not params:
+        # Project is required for quick launch
+        if not project:
+            click.echo("Project field is required if parameters file is not specified.")
+            return
+        project = get_project_id(project)
+        if not project:
+            return
+
+        # Creates web browser and generates inputs for quick_launch
+        quick_launch = _get_quick_launch(ctx, protocol_obj, project)
+
+        # Save the protocol input locally if the user specified the save_input option
+        if save_input:
+            try:
+                with click.open_file(save_input, 'w') as f:
+                    f.write(json.dumps(quick_launch["inputs"], indent=2))
+            except Exception as e:
+                print_stderr("\nUnable to save inputs: %s" % str(e))
+
+    if remote:
+        # For remote execution, use input params file if specified, else use quick_launch inputs
+        if not params:
+            params = dict(parameters=quick_launch["raw_inputs"])
+            # Save parameters to file if specified
+            if save_input:
+                try:
+                    with click.open_file(save_input, 'w') as f:
+                        f.write(json.dumps(params, indent=2))
+                except Exception as e:
+                    print_stderr("\nUnable to save inputs: %s" % str(e))
+        else:
+            try:
+                params = json.loads(params.read())
+            except ValueError:
+                print_stderr("Unable to load parameters given. "
+                             "File is probably incorrectly formatted.")
+                return
+
+        launch_protocol = _get_launch_request(ctx, params, protocol_obj)
+        from time import strftime, gmtime
+        default_title = "{}_{}".format(protocol, strftime("%b_%d_%Y", gmtime()))
+        try:
+            req_json = ctx.obj.api.submit_run(
+                launch_protocol["autoprotocol"], project_id=project, title=default_title, test_mode=None)
+            run_id = req_json['id']
+            click.echo("\nRun created: %s" %
+                       ctx.obj.api.url("%s/runs/%s" % (project, run_id)))
+        except Exception as e:
+            click.echo(str(e))
+    else:
+        print_stderr("\nGenerating Autoprotocol....\n")
+        if not params:
+            run_protocol(manifest, protocol_obj, quick_launch["inputs"])
+        else:
+            run_protocol(manifest, protocol_obj, params)
+
+
+def _create_launch_request(params, bsl=1, test_mode=False):
+    """Creates launch_request from input params"""
+    params_dict = dict()
+    params_dict["launch_request"] = params
+    params_dict["launch_request"]["bsl"] = bsl
+    params_dict["launch_request"]["test_mode"] = test_mode
+    return json.dumps(params_dict)
+
+
+def _get_launch_request(ctx, params, protocol):
+    """Launches protocol from parameters"""
+    launch_request = _create_launch_request(params)
+    launch_protocol = ctx.obj.api.launch_protocol(launch_request,
+                                                  protocol_id=protocol["id"])
+    launch_request_id = launch_protocol["id"]
+
+    # Wait until launch request is updated (max 5 minutes)
+    count = 1
+    while count <= 150 and launch_protocol['autoprotocol'] is None:
+        sys.stderr.write(
+            "\rWaiting for launch request to be configured%s" % ('.' * count))
+        sys.stderr.flush()
+        time.sleep(2)
+        launch_protocol = ctx.obj.api.get_launch_request(protocol_id=protocol["id"],
+                                                         launch_request_id=launch_request_id)
+        count += 1
+
+    return launch_protocol
+
+
+def _get_quick_launch(ctx, protocol, project):
+    """Creates quick launch object and opens it in a new tab"""
     quick_launch = ctx.obj.api.create_quick_launch(
-        data=json.dumps({"manifest": protocol}), project_id=project)
+        data=json.dumps({"manifest": protocol}),
+        project_id=project
+    )
     quick_launch_mtime = quick_launch["updated_at"]
 
     format_str = "\nOpening %s"
@@ -720,17 +954,10 @@ def launch(ctx, protocol, project, save_input):
         with stdchannel_redirected(sys.stdout, os.devnull):
             webbrowser.open_new_tab(url)
 
-    def on_json_received(protocol_inputs, error):
-        if protocol_inputs is not None:
-            print("Protocol inputs (%s)" % (protocol_inputs))
-        elif error is not None:
-            click.echo('Invalid json: %s' % e)
-            return None
-
     # Wait until the quick launch inputs are updated (max 15 minutes)
     count = 1
     while (count <= 180 and quick_launch["inputs"] is None or
-           quick_launch_mtime >= quick_launch["updated_at"]):
+                   quick_launch_mtime >= quick_launch["updated_at"]):
         sys.stderr.write(
             "\rWaiting for inputs to be configured%s" % ('.' * count))
         sys.stderr.flush()
@@ -739,24 +966,54 @@ def launch(ctx, protocol, project, save_input):
         quick_launch = ctx.obj.api.get_quick_launch(project_id=project,
                                                     quick_launch_id=quick_launch["id"])
         count += 1
+    return quick_launch
 
-    # Save the protocol input locally if the user specified the save_input
-    # option
-    if save_input:
-        try:
-            with click.open_file(save_input, 'w') as f:
-                f.write(json.dumps(quick_launch["inputs"], indent=2))
-        except Exception as e:
-            print_stderr("\nUnable to save inputs: %s" % str(e))
+@cli.command()
+@click.pass_context
+def select_org(ctx):
+    """Allows you to switch organizations"""
+    org_list = [{"name": org['name'], "subdomain": org['subdomain']} for org in ctx.obj.api.organizations()]
+    if len(org_list) < 1:
+        click.echo("Error: You don't appear to belong to any organizations. \n"
+                   "Visit {} and create an organization.".format('https://secure.transcriptic.com'))
+        sys.exit(1)
+    if len(org_list) == 1:
+        organization = org_list[0]['subdomain']
+    else:
+        click.echo("You belong to %s organizations:" %
+                   len(org_list))
+        for indx, o in enumerate(org_list):
+            click.echo("{}.  {} ({})".format(indx + 1, o['name'], o['subdomain']))
 
-    print_stderr("\nGenerating Autoprotocol....\n")
-    run_protocol(manifest, protocol, quick_launch["inputs"])
+        def parse_valid_org(indx):
+            from click.exceptions import BadParameter
+            try:
+                return org_list[int(indx) - 1]['subdomain']
+            except:
+                raise BadParameter("Please enter an integer between 1 and %s" %
+                                   (len(org_list)), ctx=ctx)
+
+        organization = click.prompt(
+            'Which organization would you like to log in as',
+            default=1,
+            prompt_suffix='? ', type=int,
+            value_proc=lambda x: parse_valid_org(x)
+        )
+
+    r = ctx.obj.api.get_organization(org_id=organization)
+    if r.status_code != 200:
+        click.echo("Error accessing organization: %s" % r.text)
+        sys.exit(1)
+
+    ctx.obj.api.organization_id = organization
+    ctx.obj.api.save(ctx.parent.params['config'])
+    click.echo('Logged in with organization: {}'.format(organization))
 
 
 @cli.command()
 @click.option('--api-root', default='https://secure.transcriptic.com')
 @click.pass_context
-def login(ctx, api_root):
+def login(ctx, api_root, analytics=True):
     """Authenticate to your Transcriptic account."""
     email = click.prompt('Email')
     password = click.prompt('Password', hide_input=True)
@@ -779,6 +1036,7 @@ def login(ctx, api_root):
     user = r.json()
     token = (user.get('authentication_token') or
              user['test_mode_authentication_token'])
+    user_id = user.get("id")
     if len(user['organizations']) < 1:
         click.echo("Error: You don't appear to belong to any organizations. \n"
                    "Visit %s and create an organization." % api_root)
@@ -792,21 +1050,21 @@ def login(ctx, api_root):
             click.echo("%s.  %s (%s)" % (indx + 1, o['name'], o['subdomain']))
 
         def parse_valid_org(indx):
+            from click.exceptions import BadParameter
             try:
                 return user['organizations'][int(indx) - 1]['subdomain']
             except:
-                click.echo("Please enter an integer between 1 and %s" %
-                           (len(user['organizations'])))
-                sys.exit(1)
+                raise BadParameter("Please enter an integer between 1 and %s" %
+                                   (len(user['organizations'])), ctx=ctx)
 
         organization = click.prompt(
-            'Which organization would you like to login as',
+            'Which organization would you like to log in as',
             default=1,
             prompt_suffix='? ', type=int,
             value_proc=lambda x: parse_valid_org(x)
         )
 
-    r = ctx.obj.api.get(routes.get_organizations(api_root=api_root, org_id=organization), headers={
+    r = ctx.obj.api.get(routes.get_organization(api_root=api_root, org_id=organization), headers={
         'X-User-Email': email,
         'X-User-Token': token,
         'Accept': 'application/json',
@@ -819,7 +1077,8 @@ def login(ctx, api_root):
         click.echo("Error accessing organization: %s" % r.text)
         sys.exit(1)
     ctx.obj.api = Connection(email=email, token=token,
-                             organization_id=organization, api_root=api_root)
+                             organization_id=organization, api_root=api_root,
+                             user_id=user_id, analytics=analytics)
     ctx.obj.api.save(ctx.parent.params['config'])
     click.echo('Logged in as %s (%s)' % (user['email'], organization))
 
