@@ -44,6 +44,18 @@ def get_packages(api_root, org_id):
     return "{api_root}/{org_id}/packages/".format(**locals())
 
 
+def get_protocols(api_root, org_id):
+    return "{api_root}/{org_id}/protocols".format(**locals())
+
+
+def launch_protocol(api_root, org_id, protocol_id):
+    return "{api_root}/{org_id}/protocols/{protocol_id}/launch".format(**locals())
+
+
+def get_launch_request(api_root, org_id, protocol_id, launch_request_id):
+    return "{api_root}/{org_id}/protocols/{protocol_id}/launch/{launch_request_id}".format(**locals())
+
+
 def post_release(api_root, org_id, package_id):
     return "{api_root}/{org_id}/packages/{package_id}/releases/".format(**locals())
 
@@ -53,13 +65,17 @@ def get_release_status(api_root, org_id, package_id, release_id, timestamp):
 
 
 def query_kits(api_root, query):
-    return "{api_root}/_commercial/kits?q={query}&per_page=1000".format(**locals())
+    return "{api_root}/_commercial/kits?q={query}&per_page=1000&full_json=true".format(**locals())
 
 
 def query_resources(api_root, query):
     return "{api_root}/_commercial/resources?q={query}&per_page=1000".format(**locals())
 
 
+def query_inventory(api_root, org_id, query, page=0):
+    return "{api_root}/{org_id}/inventory/samples?q={query}&per_page=75&page={page}".format(**locals())
+
+
 def get_quick_launch(api_root, org_id, project_id, quick_launch_id):
     return "{api_root}/{org_id}/{project_id}/runs/quick_launch/{quick_launch_id}".format(**locals())
 
@@ -80,7 +96,11 @@ def login(api_root):
     return "{api_root}/users/sign_in".format(**locals())
 
 
-def get_organizations(api_root, org_id):
+def get_organizations(api_root):
+    return "{api_root}/organizations".format(**locals())
+
+
+def get_organization(api_root, org_id):
     return "{api_root}/{org_id}".format(**locals())
 
 
@@ -128,5 +148,12 @@ def get_data_zip(api_root, data_id):
     return "{api_root}/-/{data_id}.zip".format(**locals())
 
 
-def monitoring_data(api_root, org_id, project_id, run_id, instruction_id, data_type):
-    return "{api_root}/{org_id}/{project_id}/runs/{run_id}/{instruction_id}/monitoring/{data_type}".format(**locals())
+def monitoring_data(api_root, data_type, instruction_id, grouping=None, start_time=None, end_time=None):
+    base_route = "{api_root}/sensor_data/{data_type}?instruction_id={instruction_id}".format(**locals())
+    if grouping:
+        base_route += "&grouping={grouping}".format(**locals())
+    if start_time:
+        base_route += "&start_time={start_time}".format(**locals())
+    if end_time:
+        base_route += "&end_time={end_time}".format(**locals())
+    return base_route
